# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.invoice_id import InvoiceId
from .types.create_import_invoice_request import CreateImportInvoiceRequest
from .types.import_invoice import ImportInvoice
from .types.import_invoice_update_request import ImportInvoiceUpdateRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def import_invoice(
        self, *, request: CreateImportInvoiceRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ImportInvoice:
        """
        Import an existing invoice from a third party service to reflect state in Candid.

        Parameters
        ----------
        request : CreateImportInvoiceRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import datetime

        from candid.client import CandidApiClient
        from candid.resources.import_invoice.v_1 import CreateImportInvoiceRequest
        from candid.resources.invoices.v_2 import InvoiceItemCreate, InvoiceStatus

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.import_invoice.v_1.import_invoice(
            request=CreateImportInvoiceRequest(
                patient_external_id="string",
                external_customer_identifier="string",
                note="string",
                due_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                items=[InvoiceItemCreate()],
                status=InvoiceStatus.DRAFT,
                external_identifier="string",
                customer_invoice_url="string",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/import-invoice/v1", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, invoice_id: InvoiceId, *, request_options: typing.Optional[RequestOptions] = None) -> ImportInvoice:
        """
        Retrieve and view an import invoice

        Parameters
        ----------
        invoice_id : InvoiceId
            InvoiceId to be returned

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.import_invoice.v_1.get(
            invoice_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/import-invoice/v1/{jsonable_encoder(invoice_id)}", method="GET", request_options=request_options
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        invoice_id: InvoiceId,
        *,
        request: ImportInvoiceUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImportInvoice:
        """
        Update the information on the imported invoice

        Parameters
        ----------
        invoice_id : InvoiceId

        request : ImportInvoiceUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import datetime
        import uuid

        from candid.client import CandidApiClient
        from candid.resources.import_invoice.v_1 import (
            ImportInvoiceUpdateRequest,
            InvoiceItemInfoUpdate,
            InvoiceItemUpdateType,
        )
        from candid.resources.invoices.v_2 import InvoiceItemCreate, InvoiceStatus

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.import_invoice.v_1.update(
            invoice_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ImportInvoiceUpdateRequest(
                customer_invoice_url="string",
                status=InvoiceStatus.DRAFT,
                note="string",
                due_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                items=InvoiceItemInfoUpdate(
                    update_type=InvoiceItemUpdateType.APPEND,
                    items=[InvoiceItemCreate()],
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/import-invoice/v1/{jsonable_encoder(invoice_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def import_invoice(
        self, *, request: CreateImportInvoiceRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ImportInvoice:
        """
        Import an existing invoice from a third party service to reflect state in Candid.

        Parameters
        ----------
        request : CreateImportInvoiceRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import datetime

        from candid.client import AsyncCandidApiClient
        from candid.resources.import_invoice.v_1 import CreateImportInvoiceRequest
        from candid.resources.invoices.v_2 import InvoiceItemCreate, InvoiceStatus

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.import_invoice.v_1.import_invoice(
            request=CreateImportInvoiceRequest(
                patient_external_id="string",
                external_customer_identifier="string",
                note="string",
                due_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                items=[InvoiceItemCreate()],
                status=InvoiceStatus.DRAFT,
                external_identifier="string",
                customer_invoice_url="string",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/import-invoice/v1", method="POST", json=request, request_options=request_options, omit=OMIT
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, invoice_id: InvoiceId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ImportInvoice:
        """
        Retrieve and view an import invoice

        Parameters
        ----------
        invoice_id : InvoiceId
            InvoiceId to be returned

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.import_invoice.v_1.get(
            invoice_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/import-invoice/v1/{jsonable_encoder(invoice_id)}", method="GET", request_options=request_options
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        invoice_id: InvoiceId,
        *,
        request: ImportInvoiceUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImportInvoice:
        """
        Update the information on the imported invoice

        Parameters
        ----------
        invoice_id : InvoiceId

        request : ImportInvoiceUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImportInvoice

        Examples
        --------
        import datetime
        import uuid

        from candid.client import AsyncCandidApiClient
        from candid.resources.import_invoice.v_1 import (
            ImportInvoiceUpdateRequest,
            InvoiceItemInfoUpdate,
            InvoiceItemUpdateType,
        )
        from candid.resources.invoices.v_2 import InvoiceItemCreate, InvoiceStatus

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.import_invoice.v_1.update(
            invoice_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ImportInvoiceUpdateRequest(
                customer_invoice_url="string",
                status=InvoiceStatus.DRAFT,
                note="string",
                due_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                items=InvoiceItemInfoUpdate(
                    update_type=InvoiceItemUpdateType.APPEND,
                    items=[InvoiceItemCreate()],
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/import-invoice/v1/{jsonable_encoder(invoice_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ImportInvoice, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PatientHistoryCategoryEnum(str, enum.Enum):
    PRESENT_ILLNESS = "present_illness"
    MEDICAL = "medical"
    FAMILY = "family"
    SOCIAL = "social"

    def visit(
        self,
        present_illness: typing.Callable[[], T_Result],
        medical: typing.Callable[[], T_Result],
        family: typing.Callable[[], T_Result],
        social: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatientHistoryCategoryEnum.PRESENT_ILLNESS:
            return present_illness()
        if self is PatientHistoryCategoryEnum.MEDICAL:
            return medical()
        if self is PatientHistoryCategoryEnum.FAMILY:
            return family()
        if self is PatientHistoryCategoryEnum.SOCIAL:
            return social()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from .identifier_base import IdentifierBase
from .identifier_id import IdentifierId


class Identifier(IdentifierBase):
    """
    Examples
    --------
    import uuid

    from candid import (
        Identifier,
        IdentifierCode,
        IdentifierValue_MedicareProviderIdentifier,
        State,
    )

    Identifier(
        identifier_id=uuid.UUID(
            "123e4567-e89b-12d3-a456-426614174000",
        ),
        identifier_code=IdentifierCode.MCR,
        identifier_value=IdentifierValue_MedicareProviderIdentifier(
            state=State.CA,
            provider_number="1234567890",
        ),
    )
    """

    identifier_id: IdentifierId

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validation_error import HttpRequestValidationError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from .types.organization_service_facility import OrganizationServiceFacility
from .types.organization_service_facility_create import OrganizationServiceFacilityCreate
from .types.organization_service_facility_id import OrganizationServiceFacilityId
from .types.organization_service_facility_page import OrganizationServiceFacilityPage
from .types.organization_service_facility_update import OrganizationServiceFacilityUpdate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_service_facilities.v_2.get(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacilityPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        name : typing.Optional[str]
            Filter to a name or a part of a name.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacilityPage

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_service_facilities.v_2.get_multi(
            limit=100,
            name="Test Service Facility",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/organization-service-facilities/v2",
            method="GET",
            params={"limit": limit, "name": name, "page_token": page_token},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacilityPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self, *, request: OrganizationServiceFacilityCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        request : OrganizationServiceFacilityCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        from candid import State, StreetAddressLongZip
        from candid.client import CandidApiClient
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityCreate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_service_facilities.v_2.create(
            request=OrganizationServiceFacilityCreate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.CLOSED,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
                physical_type=ServiceFacilityPhysicalType.SITE,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/organization-service-facilities/v2",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request: OrganizationServiceFacilityUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request : OrganizationServiceFacilityUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        import uuid

        from candid import State, StreetAddressLongZip
        from candid.client import CandidApiClient
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityUpdate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_service_facilities.v_2.update(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
            request=OrganizationServiceFacilityUpdate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.CLOSED,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
                physical_type=ServiceFacilityPhysicalType.SITE,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_service_facilities.v_2.delete(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.organization_service_facilities.v_2.get(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacilityPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        name : typing.Optional[str]
            Filter to a name or a part of a name.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacilityPage

        Examples
        --------
        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.organization_service_facilities.v_2.get_multi(
            limit=100,
            name="Test Service Facility",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/organization-service-facilities/v2",
            method="GET",
            params={"limit": limit, "name": name, "page_token": page_token},
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacilityPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self, *, request: OrganizationServiceFacilityCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        request : OrganizationServiceFacilityCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        from candid import State, StreetAddressLongZip
        from candid.client import AsyncCandidApiClient
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityCreate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.organization_service_facilities.v_2.create(
            request=OrganizationServiceFacilityCreate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.CLOSED,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
                physical_type=ServiceFacilityPhysicalType.SITE,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/organization-service-facilities/v2",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request: OrganizationServiceFacilityUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationServiceFacility:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request : OrganizationServiceFacilityUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationServiceFacility

        Examples
        --------
        import uuid

        from candid import State, StreetAddressLongZip
        from candid.client import AsyncCandidApiClient
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityUpdate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.organization_service_facilities.v_2.update(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
            request=OrganizationServiceFacilityUpdate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.CLOSED,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
                physical_type=ServiceFacilityPhysicalType.SITE,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        organization_service_facility_id : OrganizationServiceFacilityId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.organization_service_facilities.v_2.delete(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/organization-service-facilities/v2/{jsonable_encoder(organization_service_facility_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from .diagnosis_type_code import DiagnosisTypeCode


class DiagnosisCreate(pydantic.BaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Empty string not allowed.
    """

    code_type: DiagnosisTypeCode = pydantic.Field()
    """
    Typically, providers submitting claims to Candid are using ICD-10 diagnosis codes. If you are using ICD-10 codes, the primary diagnosis code listed on the claim should use the ABK code_type. If more than one diagnosis is being submitted on a claim, please use ABF for the rest of the listed diagnoses. If you are using ICD-9 diagnosis codes, use BK and BF for the principal and following diagnosis code(s) respectively.
    """

    code: str = pydantic.Field()
    """
    Empty string not allowed.
    Should be of the appropriate format for the provided `code_type`.
    Must obey the ICD-10 format if an ICD-10 code_type is provided, specifically:
    
    - Letter
    - Digit
    - Digit or the letter `A` or `B`
    - (Optional) Period `.`
    - Up to 4 (or as few as 0) letters and digits
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .resources import TaskCategory, TaskStatus, TaskType, commons, v_3

__all__ = ["TaskCategory", "TaskStatus", "TaskType", "commons", "v_3"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClaimAdjustmentGroupCodes(str, enum.Enum):
    CO = "CO"
    CR = "CR"
    DE = "DE"
    MA = "MA"
    OA = "OA"
    PI = "PI"
    PR = "PR"
    RR = "RR"
    NC = "NC"
    UNKNOWN = "UNKNOWN"

    def visit(
        self,
        co: typing.Callable[[], T_Result],
        cr: typing.Callable[[], T_Result],
        de: typing.Callable[[], T_Result],
        ma: typing.Callable[[], T_Result],
        oa: typing.Callable[[], T_Result],
        pi: typing.Callable[[], T_Result],
        pr: typing.Callable[[], T_Result],
        rr: typing.Callable[[], T_Result],
        nc: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClaimAdjustmentGroupCodes.CO:
            return co()
        if self is ClaimAdjustmentGroupCodes.CR:
            return cr()
        if self is ClaimAdjustmentGroupCodes.DE:
            return de()
        if self is ClaimAdjustmentGroupCodes.MA:
            return ma()
        if self is ClaimAdjustmentGroupCodes.OA:
            return oa()
        if self is ClaimAdjustmentGroupCodes.PI:
            return pi()
        if self is ClaimAdjustmentGroupCodes.PR:
            return pr()
        if self is ClaimAdjustmentGroupCodes.RR:
            return rr()
        if self is ClaimAdjustmentGroupCodes.NC:
            return nc()
        if self is ClaimAdjustmentGroupCodes.UNKNOWN:
            return unknown()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ServiceFacilityPhysicalType(str, enum.Enum):
    """
    Examples
    --------
    from candid.resources.organization_service_facilities.v_2 import (
        ServiceFacilityPhysicalType,
    )

    ServiceFacilityPhysicalType.SITE
    """

    SITE = "si"
    """
    A collection of buildings or other locations such as a site or a campus.
    """

    BUILDING = "bu"
    """
    Any Building or structure. This may contain rooms, corridors, wings, etc. It might not have walls, or a roof, but is considered a defined/allocated space.
    """

    WING = "wi"
    """
    A Wing within a Building, this often contains levels, rooms and corridors.
    """

    WARD = "wa"
    """
    A Ward is a section of a medical facility that may contain rooms and other types of location.
    """

    LEVEL = "lvl"
    """
    A Level in a multi-level Building/Structure.
    """

    CORRIDOR = "co"
    """
    A Corridor within a Building.
    """

    ROOM = "ro"
    """
    A space that is allocated as a room, it may have walls/roof etc., but does not require these.
    """

    BED = "bd"
    """
    A space that is allocated for sleeping/laying on etc. typically within a building, section of a building or within a room.
    """

    VEHICLE = "ve"
    """
    A means of transportation.
    """

    HOUSE = "ho"
    """
    A residential dwelling. Usually used to reference a location that a person/patient may reside.
    """

    CABINET = "ca"
    """
    A container that can store goods, equipment, medications or other items.
    """

    ROAD = "rd"
    """
    A defined path to travel between 2 points that has a known name.
    """

    AREA = "area"
    """
    A defined physical boundary of something, such as a flood risk zone, region, postcode.
    """

    JURISDICTION = "jdn"
    """
    A wide scope that covers a conceptual domain, such as a Nation (Country wide community or Federal Government - e.g. Ministry of Health), Province or State (community or Government), Business (throughout the enterprise), Nation with a business scope of an agency (e.g. CDC, FDA etc.) or a Business segment (UK Pharmacy), not just an physical boundary.
    """

    def visit(
        self,
        site: typing.Callable[[], T_Result],
        building: typing.Callable[[], T_Result],
        wing: typing.Callable[[], T_Result],
        ward: typing.Callable[[], T_Result],
        level: typing.Callable[[], T_Result],
        corridor: typing.Callable[[], T_Result],
        room: typing.Callable[[], T_Result],
        bed: typing.Callable[[], T_Result],
        vehicle: typing.Callable[[], T_Result],
        house: typing.Callable[[], T_Result],
        cabinet: typing.Callable[[], T_Result],
        road: typing.Callable[[], T_Result],
        area: typing.Callable[[], T_Result],
        jurisdiction: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ServiceFacilityPhysicalType.SITE:
            return site()
        if self is ServiceFacilityPhysicalType.BUILDING:
            return building()
        if self is ServiceFacilityPhysicalType.WING:
            return wing()
        if self is ServiceFacilityPhysicalType.WARD:
            return ward()
        if self is ServiceFacilityPhysicalType.LEVEL:
            return level()
        if self is ServiceFacilityPhysicalType.CORRIDOR:
            return corridor()
        if self is ServiceFacilityPhysicalType.ROOM:
            return room()
        if self is ServiceFacilityPhysicalType.BED:
            return bed()
        if self is ServiceFacilityPhysicalType.VEHICLE:
            return vehicle()
        if self is ServiceFacilityPhysicalType.HOUSE:
            return house()
        if self is ServiceFacilityPhysicalType.CABINET:
            return cabinet()
        if self is ServiceFacilityPhysicalType.ROAD:
            return road()
        if self is ServiceFacilityPhysicalType.AREA:
            return area()
        if self is ServiceFacilityPhysicalType.JURISDICTION:
            return jurisdiction()



# This file was auto-generated by Fern from our API Definition.

from .types import PatientPayment, PatientPaymentId, PatientPaymentSource, PatientPaymentStatus

__all__ = ["PatientPayment", "PatientPaymentId", "PatientPaymentSource", "PatientPaymentStatus"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PatientTransactionSource(str, enum.Enum):
    MANUAL_ENTRY = "MANUAL_ENTRY"
    CHARGEBEE = "CHARGEBEE"
    SQUARE = "SQUARE"
    STRIPE = "STRIPE"
    ELATION = "ELATION"
    CEDAR = "CEDAR"
    HEALTHIE = "HEALTHIE"

    def visit(
        self,
        manual_entry: typing.Callable[[], T_Result],
        chargebee: typing.Callable[[], T_Result],
        square: typing.Callable[[], T_Result],
        stripe: typing.Callable[[], T_Result],
        elation: typing.Callable[[], T_Result],
        cedar: typing.Callable[[], T_Result],
        healthie: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatientTransactionSource.MANUAL_ENTRY:
            return manual_entry()
        if self is PatientTransactionSource.CHARGEBEE:
            return chargebee()
        if self is PatientTransactionSource.SQUARE:
            return square()
        if self is PatientTransactionSource.STRIPE:
            return stripe()
        if self is PatientTransactionSource.ELATION:
            return elation()
        if self is PatientTransactionSource.CEDAR:
            return cedar()
        if self is PatientTransactionSource.HEALTHIE:
            return healthie()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AccountType(str, enum.Enum):
    PATIENT = "PATIENT"
    INSURANCE = "INSURANCE"

    def visit(self, patient: typing.Callable[[], T_Result], insurance: typing.Callable[[], T_Result]) -> T_Result:
        if self is AccountType.PATIENT:
            return patient()
        if self is AccountType.INSURANCE:
            return insurance()



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from .errors.too_many_requests_error import TooManyRequestsError
from .types.auth_get_token_response import AuthGetTokenResponse
from .types.too_many_requests_error_type import TooManyRequestsErrorType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthGetTokenResponse:
        """
        Authenticating with the Candid Health API.

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        **Warning:**

        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an "HTTP 429 Too Many Requests" error.

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthGetTokenResponse

        Examples
        --------
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.auth.v_2.get_token(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/auth/v2/token",
            method="POST",
            json={"client_id": client_id, "client_secret": client_secret},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AuthGetTokenResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    pydantic_v1.parse_obj_as(TooManyRequestsErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_token(
        self, *, client_id: str, client_secret: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AuthGetTokenResponse:
        """
        Authenticating with the Candid Health API.

        Candid Health utilizes the [OAuth 2.0 bearer token authentication scheme](https://developer.mozilla.org/en-US/docs/Web/HTTP/Authentication) in our auth flow. You obtain the bearer token for all
        subsequent API requests via the `/auth/token` endpoint defined below, which requires you to provide your `client_id` and `client_secret`. Your `client_id` and `client_secret` can be [generated](https://support.joincandidhealth.com/hc/en-us/articles/23065219476244--Generating-Candid-API-Keys) from the "Users & Credentials" tab by your org admin.

        The bearer token should be provided in the `Authorization` header for all subsequent API calls.

        **Warning:**

        The bearer token expires 5 hours after it has been created. After it has expired, the client will receive an "HTTP 401
        Unauthorized" error, at which point the client should generate a new token. It is important that tokens be reused between requests; if the client attempts to generate a token too often, it will be rate-limited and will receive an "HTTP 429 Too Many Requests" error.

        Parameters
        ----------
        client_id : str
            Your application's Client ID.

        client_secret : str
            Your application's Client Secret.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthGetTokenResponse

        Examples
        --------
        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.auth.v_2.get_token(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/auth/v2/token",
            method="POST",
            json={"client_id": client_id, "client_secret": client_secret},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(AuthGetTokenResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    pydantic_v1.parse_obj_as(TooManyRequestsErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.types.encounter_id import EncounterId
from .types.billing_note import BillingNote

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, encounter_id: EncounterId, text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> BillingNote:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        text : str
            Empty string not allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingNote

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.billing_notes.v_2.create(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            text="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/billing_notes/v2",
            method="POST",
            json={"encounter_id": encounter_id, "text": text},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, encounter_id: EncounterId, text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> BillingNote:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        text : str
            Empty string not allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingNote

        Examples
        --------
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.billing_notes.v_2.create(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            text="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/billing_notes/v2",
            method="POST",
            json={"encounter_id": encounter_id, "text": text},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.types.date import Date
from ....commons.types.insurance_type_code import InsuranceTypeCode
from ....commons.types.state import State
from .types.expected_network_status_response import ExpectedNetworkStatusResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def compute(
        self,
        *,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ExpectedNetworkStatusResponse:
        """
        Computes the expected network status given the provided information.

        Parameters
        ----------
        subscriber_payer_id : str

        subscriber_payer_name : str

        billing_provider_npi : str
            The National Provider Identifier (NPI) of the healthcare provider responsible for billing. A unique 10-digit identification number.

        billing_provider_tin : str
            Follow the 9-digit format of the Taxpayer Identification Number (TIN).

        rendering_provider_npi : str
            The National Provider Identifier (NPI) of the healthcare provider who delivered the services. A unique 10-digit identification number.

        contracted_state : State
            The state in which the healthcare provider has a contractual agreement with the insurance payer.

        date_of_service : Date
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.


        external_patient_id : typing.Optional[str]

        subscriber_insurance_type : typing.Optional[InsuranceTypeCode]

        subscriber_plan_name : typing.Optional[str]
            The descriptive name of the insurance plan selected by the subscriber, often indicating coverage specifics or tier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpectedNetworkStatusResponse

        Examples
        --------
        from candid import InsuranceTypeCode, State
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.expected_network_status.v_1.compute(
            external_patient_id="string",
            subscriber_payer_id="string",
            subscriber_payer_name="string",
            subscriber_insurance_type=InsuranceTypeCode.C_01,
            subscriber_plan_name="string",
            billing_provider_npi="string",
            billing_provider_tin="string",
            rendering_provider_npi="string",
            contracted_state=State.AA,
            date_of_service="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/expected-network-status/v1",
            method="POST",
            json={
                "external_patient_id": external_patient_id,
                "subscriber_payer_id": subscriber_payer_id,
                "subscriber_payer_name": subscriber_payer_name,
                "subscriber_insurance_type": subscriber_insurance_type,
                "subscriber_plan_name": subscriber_plan_name,
                "billing_provider_npi": billing_provider_npi,
                "billing_provider_tin": billing_provider_tin,
                "rendering_provider_npi": rendering_provider_npi,
                "contracted_state": contracted_state,
                "date_of_service": date_of_service,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def compute(
        self,
        *,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ExpectedNetworkStatusResponse:
        """
        Computes the expected network status given the provided information.

        Parameters
        ----------
        subscriber_payer_id : str

        subscriber_payer_name : str

        billing_provider_npi : str
            The National Provider Identifier (NPI) of the healthcare provider responsible for billing. A unique 10-digit identification number.

        billing_provider_tin : str
            Follow the 9-digit format of the Taxpayer Identification Number (TIN).

        rendering_provider_npi : str
            The National Provider Identifier (NPI) of the healthcare provider who delivered the services. A unique 10-digit identification number.

        contracted_state : State
            The state in which the healthcare provider has a contractual agreement with the insurance payer.

        date_of_service : Date
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.


        external_patient_id : typing.Optional[str]

        subscriber_insurance_type : typing.Optional[InsuranceTypeCode]

        subscriber_plan_name : typing.Optional[str]
            The descriptive name of the insurance plan selected by the subscriber, often indicating coverage specifics or tier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpectedNetworkStatusResponse

        Examples
        --------
        from candid import InsuranceTypeCode, State
        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.expected_network_status.v_1.compute(
            external_patient_id="string",
            subscriber_payer_id="string",
            subscriber_payer_name="string",
            subscriber_insurance_type=InsuranceTypeCode.C_01,
            subscriber_plan_name="string",
            billing_provider_npi="string",
            billing_provider_tin="string",
            rendering_provider_npi="string",
            contracted_state=State.AA,
            date_of_service="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/expected-network-status/v1",
            method="POST",
            json={
                "external_patient_id": external_patient_id,
                "subscriber_payer_id": subscriber_payer_id,
                "subscriber_payer_name": subscriber_payer_name,
                "subscriber_insurance_type": subscriber_insurance_type,
                "subscriber_plan_name": subscriber_plan_name,
                "billing_provider_npi": billing_provider_npi,
                "billing_provider_tin": billing_provider_tin,
                "rendering_provider_npi": rendering_provider_npi,
                "contracted_state": contracted_state,
                "date_of_service": date_of_service,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ClaimInvoiceItem,
    ClaimInvoiceItemInfo,
    Invoice,
    InvoiceDestination,
    InvoiceDestinationMetadata,
    InvoiceInfo,
    InvoiceItemAttributionCreate,
    InvoiceItemAttributionCreate_ClaimId,
    InvoiceItemAttributionCreate_ServiceLineId,
    InvoiceItemAttributionCreate_Unattributed,
    InvoiceItemCreate,
    InvoiceItemInfo,
    InvoiceStatus,
    ServiceLineInvoiceItem,
    UnattributedInvoiceItem,
)

__all__ = [
    "ClaimInvoiceItem",
    "ClaimInvoiceItemInfo",
    "Invoice",
    "InvoiceDestination",
    "InvoiceDestinationMetadata",
    "InvoiceInfo",
    "InvoiceItemAttributionCreate",
    "InvoiceItemAttributionCreate_ClaimId",
    "InvoiceItemAttributionCreate_ServiceLineId",
    "InvoiceItemAttributionCreate_Unattributed",
    "InvoiceItemCreate",
    "InvoiceItemInfo",
    "InvoiceStatus",
    "ServiceLineInvoiceItem",
    "UnattributedInvoiceItem",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...commons.types.date import Date
from ...commons.types.invoice_id import InvoiceId
from ...commons.types.organization_id import OrganizationId
from ...commons.types.patient_external_id import PatientExternalId
from .invoice_item import InvoiceItem
from .invoice_status import InvoiceStatus


class Invoice(pydantic.BaseModel):
    id: InvoiceId
    created_at: dt.datetime
    updated_at: dt.datetime
    organzation_id: OrganizationId
    source_id: str
    source_customer_id: str
    patient_external_id: PatientExternalId
    note: typing.Optional[str] = None
    due_date: Date
    status: InvoiceStatus
    url: typing.Optional[str] = None
    customer_invoice_url: typing.Optional[str] = None
    items: typing.List[InvoiceItem]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ResponsiblePartyType(str, enum.Enum):
    INSURANCE_PAY = "INSURANCE_PAY"
    SELF_PAY = "SELF_PAY"
    UNKNOWN = "UNKNOWN"

    def visit(
        self,
        insurance_pay: typing.Callable[[], T_Result],
        self_pay: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ResponsiblePartyType.INSURANCE_PAY:
            return insurance_pay()
        if self is ResponsiblePartyType.SELF_PAY:
            return self_pay()
        if self is ResponsiblePartyType.UNKNOWN:
            return unknown()



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .resources.v_2.client import AsyncV2Client, V2Client


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.v_2 = V2Client(client_wrapper=self._client_wrapper)


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.v_2 = AsyncV2Client(client_wrapper=self._client_wrapper)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.date import Date
from .....commons.types.delay_reason_code import DelayReasonCode
from .....commons.types.encounter_external_id import EncounterExternalId
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .billable_status_type import BillableStatusType
from .intervention import Intervention
from .medication import Medication
from .prior_authorization_number import PriorAuthorizationNumber
from .responsible_party_type import ResponsiblePartyType
from .service_authorization_exception_code import ServiceAuthorizationExceptionCode
from .synchronicity_type import SynchronicityType
from .vitals import Vitals


class EncounterBase(pydantic.BaseModel):
    external_id: EncounterExternalId = pydantic.Field()
    """
    A client-specified unique ID to associate with this encounter;
    for example, your internal encounter ID or a Dr. Chrono encounter ID.
    This field should not contain PHI.
    """

    prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = pydantic.Field(default=None)
    """
    Box 23 on the CMS-1500 claim form.
    """

    patient_authorized_release: bool = pydantic.Field()
    """
    Whether this patient has authorized the release of medical information
    for billing purpose.
    Box 12 on the CMS-1500 claim form.
    """

    benefits_assigned_to_provider: bool = pydantic.Field()
    """
    Whether this patient has authorized insurance payments to be made to you,
    not them. If false, patient may receive reimbursement.
    Box 13 on the CMS-1500 claim form.
    """

    provider_accepts_assignment: bool = pydantic.Field()
    """
    Whether you have accepted the patient's authorization for insurance payments
    to be made to you, not them.
    Box 27 on the CMS-1500 claim form.
    """

    appointment_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable description of the appointment type (ex: "Acupuncture - Headaches").
    """

    existing_medications: typing.Optional[typing.List[Medication]] = None
    vitals: typing.Optional[Vitals] = None
    interventions: typing.Optional[typing.List[Intervention]] = None
    pay_to_address: typing.Optional[StreetAddressLongZip] = pydantic.Field(default=None)
    """
    Specifies the address to which payments for the claim should be sent.
    """

    synchronicity: typing.Optional[SynchronicityType] = pydantic.Field(default=None)
    """
    Whether or not this was a synchronous or asynchronous encounter.
    Asynchronous encounters occur when providers and patients communicate online using
    forms, instant messaging, or other pre-recorded digital mediums.
    Synchronous encounters occur in live, real-time settings where the patient interacts
    directly with the provider, such as over video or a phone call.
    """

    billable_status: BillableStatusType = pydantic.Field()
    """
    Defines if the Encounter is to be billed by Candid to the responsible_party.
    Examples for when this should be set to NOT_BILLABLE include
    if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.
    """

    responsible_party: ResponsiblePartyType = pydantic.Field()
    """
    Defines the party to be billed with the initial balance owed on the claim.
    """

    additional_information: typing.Optional[str] = pydantic.Field(default=None)
    """
    Defines additional information on the claim needed by the payer.
    Box 19 on the CMS-1500 claim form.
    """

    service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = pydantic.Field(
        default=None
    )
    """
    837p Loop2300 REF\*4N
    Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
    reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
    obtaining the authorization.
    """

    admission_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP\*435, CMS-1500 Box 18
    Required on all ambulance claims when the patient was known to be admitted to the hospital.
    OR
    Required on all claims involving inpatient medical visits.
    """

    discharge_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP\*096, CMS-1500 Box 18
    Required for inpatient claims when the patient was discharged from the facility and the discharge date is known.
    """

    onset_of_current_illness_or_symptom_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP\*431, CMS-1500 Box 14
    Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
    OR
    This date is the onset of acute symptoms for the current illness or condition.
    """

    last_menstrual_period_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    837p Loop2300 DTP\*484, CMS-1500 Box 14
    Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.
    """

    delay_reason_code: typing.Optional[DelayReasonCode] = pydantic.Field(default=None)
    """
    837i Loop2300, CLM-1300 Box 20
    Code indicating the reason why a request was delayed
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.api_error import ApiError
from ..types.rate_upload_with_possible_errors import RateUploadWithPossibleErrors


class FeeScheduleValidationHttpError(ApiError):
    def __init__(self, body: typing.List[RateUploadWithPossibleErrors]):
        super().__init__(status_code=422, body=body)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InsuranceWriteOffReason(str, enum.Enum):
    SMALL_BALANCE = "SMALL_BALANCE"
    NO_AUTHORIZATION_REFERRAL = "NO_AUTHORIZATION_REFERRAL"
    TIMELY_FILING = "TIMELY_FILING"
    STALE_DATE = "STALE_DATE"
    TIMELY_FILING_LATE_ENCOUNTER = "TIMELY_FILING_LATE_ENCOUNTER"
    CREDENTIALING_OR_CONTRACTING = "CREDENTIALING_OR_CONTRACTING"
    NON_COVERED_MAX_BENEFIT = "NON_COVERED_MAX_BENEFIT"
    NOT_MEDICALLY_NECESSARY = "NOT_MEDICALLY_NECESSARY"
    BUNDLED_OR_INCLUSIVE = "BUNDLED_OR_INCLUSIVE"
    UNCOLLECTIBLE_OR_NON_BILLABLE = "UNCOLLECTIBLE_OR_NON_BILLABLE"
    EFFORTS_EXHAUSTED = "EFFORTS_EXHAUSTED"
    ADMINISTRATIVE_WRITE_OFF = "ADMINISTRATIVE_WRITE_OFF"
    CASE_RATE_OR_CAPITATED = "CASE_RATE_OR_CAPITATED"
    OTHER = "OTHER"
    UNKNOWN = "UNKNOWN"
    CONTRACTUAL_ADJUSTMENT = "CONTRACTUAL_ADJUSTMENT"

    def visit(
        self,
        small_balance: typing.Callable[[], T_Result],
        no_authorization_referral: typing.Callable[[], T_Result],
        timely_filing: typing.Callable[[], T_Result],
        stale_date: typing.Callable[[], T_Result],
        timely_filing_late_encounter: typing.Callable[[], T_Result],
        credentialing_or_contracting: typing.Callable[[], T_Result],
        non_covered_max_benefit: typing.Callable[[], T_Result],
        not_medically_necessary: typing.Callable[[], T_Result],
        bundled_or_inclusive: typing.Callable[[], T_Result],
        uncollectible_or_non_billable: typing.Callable[[], T_Result],
        efforts_exhausted: typing.Callable[[], T_Result],
        administrative_write_off: typing.Callable[[], T_Result],
        case_rate_or_capitated: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        contractual_adjustment: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is InsuranceWriteOffReason.SMALL_BALANCE:
            return small_balance()
        if self is InsuranceWriteOffReason.NO_AUTHORIZATION_REFERRAL:
            return no_authorization_referral()
        if self is InsuranceWriteOffReason.TIMELY_FILING:
            return timely_filing()
        if self is InsuranceWriteOffReason.STALE_DATE:
            return stale_date()
        if self is InsuranceWriteOffReason.TIMELY_FILING_LATE_ENCOUNTER:
            return timely_filing_late_encounter()
        if self is InsuranceWriteOffReason.CREDENTIALING_OR_CONTRACTING:
            return credentialing_or_contracting()
        if self is InsuranceWriteOffReason.NON_COVERED_MAX_BENEFIT:
            return non_covered_max_benefit()
        if self is InsuranceWriteOffReason.NOT_MEDICALLY_NECESSARY:
            return not_medically_necessary()
        if self is InsuranceWriteOffReason.BUNDLED_OR_INCLUSIVE:
            return bundled_or_inclusive()
        if self is InsuranceWriteOffReason.UNCOLLECTIBLE_OR_NON_BILLABLE:
            return uncollectible_or_non_billable()
        if self is InsuranceWriteOffReason.EFFORTS_EXHAUSTED:
            return efforts_exhausted()
        if self is InsuranceWriteOffReason.ADMINISTRATIVE_WRITE_OFF:
            return administrative_write_off()
        if self is InsuranceWriteOffReason.CASE_RATE_OR_CAPITATED:
            return case_rate_or_capitated()
        if self is InsuranceWriteOffReason.OTHER:
            return other()
        if self is InsuranceWriteOffReason.UNKNOWN:
            return unknown()
        if self is InsuranceWriteOffReason.CONTRACTUAL_ADJUSTMENT:
            return contractual_adjustment()



# This file was auto-generated by Fern from our API Definition.

from .create_import_invoice_request import CreateImportInvoiceRequest
from .import_invoice import ImportInvoice
from .import_invoice_update_request import ImportInvoiceUpdateRequest
from .invoice_item_info_update import InvoiceItemInfoUpdate
from .invoice_item_update_type import InvoiceItemUpdateType

__all__ = [
    "CreateImportInvoiceRequest",
    "ImportInvoice",
    "ImportInvoiceUpdateRequest",
    "InvoiceItemInfoUpdate",
    "InvoiceItemUpdateType",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ServiceFacilityType(str, enum.Enum):
    """
    Examples
    --------
    from candid.resources.organization_service_facilities.v_2 import (
        ServiceFacilityType,
    )

    ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT
    """

    DIAGNOSTICS_OR_THERAPEUTICS_UNIT = "DX"
    """
    A practice setting where diagnostic procedures or therapeutic interventions are performed.
    """

    CARDIOVASCULAR_DIAGNOSTICS_OR_THERAPEUTICS_UNIT = "CVDX"
    """
    A practice setting where cardiovascular diagnostic procedures or therapeutic interventions are performed.
    """

    CARDIAC_CATHETERIZATION_LAB = "CATH"
    """
    Cardiac catheterization lab.
    """

    ECHOCARDIOGRAPHY_LAB = "ECHO"
    """
    Echocardiography lab.
    """

    GASTROENTEROLOGY_DIAGNOSTICS_OR_THERAPEUTICS_LAB = "GIDX"
    """
    A practice setting where GI procedures (such as endoscopies) are performed.
    """

    ENDOSCOPY_LAB = "ENDOS"
    """
    (X12N 261QD0000N)
    """

    RADIOLOGY_DIAGNOSTICS_OR_THERAPEUTICS_UNIT = "RADDX"
    """
    A practice setting where radiology services (diagnostic or therapeutic) are provided (X12N 261QR0200N).
    """

    RADIATION_ONCOLOGY_UNIT = "RADO"
    """
    (X12N 261QX0203N)
    """

    NEURORADIOLOGY_UNIT = "RNEU"
    """
    Neuroradiology unit.
    """

    HOSPITAL = "HOSP"
    """
    An acute care institution that provides medical, surgical, or psychiatric care and treatment for the sick or the injured.
    """

    CHRONIC_CARE_FACILITY = "CHR"
    """
    (1) A hospital including a physical plant and personnel that provides multidisciplinary diagnosis and treatment for diseases that have one or more of the following characteristics: is permanent; leaves residual disability; is caused by nonreversible pathological alteration; requires special training of the patient for rehabilitation; and/or may be expected to require a long period of supervision or care. In addition, patients require the safety, security, and shelter of these specialized inpatient or partial hospitalization settings. (2) A hospital that provides medical and skilled nursing services to patients with long-term illnesses who are not in an acute phase but who require an intensity of services not available in nursing homes.
    """

    HOSPITALS_GENERAL_ACUTE_CARE_HOSPITAL = "GACH"
    """
    (X12N 282N00000N)
    """

    MILITARY_HOSPITAL = "MHSP"
    """
    A health care facility operated by the Department of Defense or other military operation.
    """

    PSYCHATRIC_CARE_FACILITY = "PSYCHF"
    """
    Healthcare facility that cares for patients with psychiatric illness(s).
    """

    REHABILITATION_HOSPITAL = "RH"
    """
    (X12N 283X00000N)
    """

    ADDICTION_TREATMENT_CENTER = "RHAT"
    """
    A location that plays the role of delivering services which may include life training and/or social support to people with addictions.
    """

    INTELLECTUAL_IMPAIRMENT_CENTER = "RHII"
    """
    A location that plays the role of delivering services which may include adaptation, rehabilitation and social integration services for people with intellectual and/or pervasive development disorders such as autism or severe behaviour disorder.
    """

    PARENTS_WITH_ADJUSTMENT_DIFFICULTIES_CENTER = "RHMAD"
    """
    A location that plays the role of delivering services which may social support services for adolescents who are pregnant or have child and are experiencing adaptation issues/difficulties in their current or eventual parenting role.
    """

    PHYSICAL_IMPAIRMENT_CENTER = "RHPI"
    """
    A location that plays the role of delivering services which may include adaptation, rehabilitation and social integration services for people with physical impairments.
    """

    PHYSICAL_IMPAIRMENT_HEARING_CENTER = "RHPIH"
    """
    A location that plays the role of delivering services for people with hearing impairments.
    """

    PHYSICAL_IMPAIRMENT_MOTOR_SKILLS_CENTER = "RHPIMS"
    """
    A location that plays the role of delivering services for people with motor skill impairments.
    """

    PHYSICAL_IMPAIRMENT_VISUAL_SKILLS_CENTER = "RHPIVS"
    """
    A location that plays the role of delivering services for people with visual skill impairments.
    """

    YOUTHS_WITH_ADJUSTMENT_DIFFICULTIES_CENTER = "RHYAD"
    """
    A location that plays the role of delivering services which may include life training and/or social support services for the adaptation, rehabilitation and social integration of youths with adjustment difficulties.
    """

    HOSPITAL_UNIT = "HU"
    """
    Hospital unit.
    """

    BONE_MARROW_TRANSPLANT_UNIT = "BMTU"
    """
    Bone marrow transplant unit.
    """

    CORONARY_CARE_UNIT = "CCU"
    """
    Coronary care unit.
    """

    CHEST_UNIT = "CHEST"
    """
    A specialty unit in hospital that focuses on chronic respirator patients. and pulmonary failure
    """

    EPILEPSY_UNIT = "EPIL"
    """
    Epilepsy unit.
    """

    EMERGENCY_ROOM = "ER"
    """
    The section of a health care facility for providing rapid treatment to victims of sudden illness or trauma.
    """

    EMERGENCY_TRAUMA_UNIT = "ETU"
    """
    Emergency trauma unit.
    """

    HEMODIALYSIS_UNIT = "HD"
    """
    Hemodialysis unit.
    """

    HOSPITAL_LABORATORY = "HLAB"
    """
    A location that plays the role of delivering services which may include tests done based on clinical specimens to get health information about a patient as pertaining to the diagnosis"
    """

    INPATIENT_LABORATORY = "INLAB"
    """
    A location that plays the role of delivering services which may include tests are done on clinical specimens to get health information about a patient pertaining to the diagnosis.
    """

    OUTPATIENT_LABORATORY = "OUTLAB"
    """
    A location that plays the role of delivering services which may include tests are done on clinical specimens to get health information about a patient pertaining to the diagnosis
    """

    RADIOLOGY_UNIT = "HRAD"
    """
    A location that plays the role of delivering services which may include the branch of medicine that uses ionizing and non-ionizing radiation to diagnose and treat diseases. The radiology unit may be further dividedinto subspecialties such as Imaging, Cardiovascular, Thoracic, and Ultrasound.
    """

    SPECIMEN_COLLECTION_SITE = "HUSCS"
    """
    A location that plays the role of delivering services which may include collecting specimens and/or samples from patients for laboratory testing purposes.
    """

    INTENSIVE_CARE_UNIT = "ICU"
    """
    Intensive care unit.
    """

    PEDIATRIC_INTENSIVE_CARE_UNIT = "PEDICU"
    """
    Pediatric intensive care unit
    """

    PEDIATRIC_NEONATAL_INTENSIVE_CARE_UNIT = "PEDNICU"
    """
    Pediatric neonatal intensive care unit.
    """

    INPATIENT_PHARMACY = "INPHARM"
    """
    A location that plays the role of delivering services which may include providing judicious, safe, efficacious, appropriate and cost effective use of medicines for treatment of patients for visits longer than one day. The distinction between inpatient pharmacies and retail (or outpatient) pharmacies is that they are part of a patient's continuity of care while staying in the hospital.
    """

    MEDICAL_LABORATORY = "MBL"
    """
    A location that plays the role of delivering services which include biochemistry, hematology, microbiology, immunochemistry, and toxicology.
    """

    NEUROLOGY_CRITICAL_CARE_AND_STROKE_UNIT = "NCCS"
    """
    Neurology critical care and stroke unit.
    """

    NEUROSURGERY_UNIT = "NS"
    """
    Neurosurgery unit.
    """

    OUTPATIENT_PHARMACY = "OUTPHARM"
    """
    A location that plays the role of delivering services which may include providing judicious, safe, efficacious, appropriate and cost effective use of medicines for treatment of patients for outpatient visits and may also be used for discharge prescriptions.
    """

    PEDIATRIC_UNIT = "PEDU"
    """
    Pediatric unit.
    """

    PSYCHIATRIC_HOSPITAL_UNIT = "PHU"
    """
    (X12N 273R00000N)
    """

    REHABILITATION_HOSPITAL_UNIT = "RHU"
    """
    Rehabilitation hospital unit.
    """

    SLEEP_DISORDERS_UNIT = "SLEEP"
    """
    (X12N 261QA1200N)
    """

    NURSING_OR_CUSTODIAL_CARE_FACILITY = "NCCF"
    """
    Nursing or custodial care facility.
    """

    SKILLED_NURSING_FACILITY = "SNF"
    """
    (X12N 314000000N)
    """

    OUTPATIENT_FACILITY = "OF"
    """
    Outpatient facility.
    """

    ALLERGY_CLINIC = "ALL"
    """
    Allergy clinic.
    """

    AMPUTEE_CLINIC = "AMPUT"
    """
    Amputee clinic.
    """

    BONE_MARROW_TRANSPLANT_CLINIC = "BMTC"
    """
    Bone marrow transplant clinic.
    """

    BREAST_CLINIC = "BREAST"
    """
    Breast clinic.
    """

    CHILD_AND_ADOLESCENT_NEUROLOGY_CLINIC = "CANC"
    """
    Child and adolescent neurology clinic.
    """

    CHILD_AND_ADOLESCENT_PSYCHIATRY_CLINIC = "CAPC"
    """
    Child and adolescent psychiatry clinic.
    """

    AMBULATORY_HEALTH_CARE_FACILITIES_CLINIC_CENTER_REHABILITATION_CARDIAC_FACILITIES = "CARD"
    """
    Ambulatory Health Care Facilities; Clinic/Center; Rehabilitation: Cardiac Facilities.
    """

    PEDIATRIC_CARDIOLOGY_CLINIC = "PEDCARD"
    """
    Pediatric cardiology clinic.
    """

    COAGULATION_CLINIC = "COAG"
    """
    Coagulation clinic.
    """

    COLON_AND_RECTAL_SURGERY_CLINIC = "CRS"
    """
    Colon and rectal surgery clinic.
    """

    DERMATOLOGY_CLINIC = "DERM"
    """
    Dermatology clinic.
    """

    ENDOCRINOLOGY_CLINIC = "ENDO"
    """
    Endocrinology clinic.
    """

    PEDIATRIC_ENDOCRINOLOGY_CLINIC = "PEDE"
    """
    Pediatric endocrinology clinic.
    """

    OTORHINOLARYNGOLOGY_CLINIC = "ENT"
    """
    Otorhinolaryngology clinic.
    """

    FAMILY_MEDICINE_CLINIC = "FMC"
    """
    Family medicine clinic.
    """

    GASTROENTEROLOGY_CLINIC = "GI"
    """
    Gastroenterology clinic.
    """

    PEDIATRIC_GASTROENTEROLOGY_CLINIC = "PEDGI"
    """
    Pediatric gastroenterology clinic.
    """

    GENERAL_INTERNAL_MEDICINE_CLINIC = "GIM"
    """
    General internal medicine clinic.
    """

    GYNECOLOGY_CLINIC = "GYN"
    """
    Gynecology clinic.
    """

    HEMATOLOGY_CLINIC = "HEM"
    """
    Hematology clinic.
    """

    PEDIATRIC_HEMATOLOGY_CLINIC = "PEDHEM"
    """
    Pediatric hematology clinic.
    """

    HYPERTENSION_CLINIC = "HTN"
    """
    Hypertension clinic.
    """

    IMPAIRMENT_EVALUATION_CENTER = "IEC"
    """
    Focuses on assessing disability.
    """

    INFECTIOUS_DISEASE_CLINIC = "INFD"
    """
    Infectious disease clinic.
    """

    PEDIATRIC_INFECTIOUS_DISEASE_CLINIC = "PEDID"
    """
    Pediatric infectious disease clinic.
    """

    INFERTILITY_CLINIC = "INV"
    """
    Infertility clinic.
    """

    LYMPEDEMA_CLINIC = "LYMPH"
    """
    Lympedema clinic.
    """

    MEDICAL_GENETICS_CLINIC = "MGEN"
    """
    Medical genetics clinic.
    """

    NEPHROLOGY_CLINIC = "NEPH"
    """
    Nephrology clinic.
    """

    PEDIATRIC_NEPHROLOGY_CLINIC = "PEDNEPH"
    """
    Pediatric nephrology clinic.
    """

    NEUROLOGY_CLINIC = "NEUR"
    """
    Neurology clinic.
    """

    OBSTETRICS_CLINIC = "OB"
    """
    Obstetrics clinic.
    """

    ORAL_AND_MAXILLOFACIAL_SURGERY_CLINIC = "OMS"
    """
    Oral and maxillofacial surgery clinic.
    """

    MEDICAL_ONCOLOGY_CLINIC = "ONCL"
    """
    Medical oncology clinic.
    """

    PEDIATRIC_ONCOLOGY_CLINIC = "PEDHO"
    """
    Pediatric oncology clinic.
    """

    OPTHALMOLOGY_CLINIC = "OPH"
    """
    Opthalmology clinic.
    """

    OPTOMETRY_CLINIC = "OPTC"
    """
    A location that plays the role of delivering services which may include examination, diagnosis, treatment, management, and prevention of diseases and disorders of the eye as well as prescribing and fitting appropriate corrective lenses (glasses or contact lenses) as needed. Optometry clinics may also provide tests for visual field screening, measuring intra-ocular pressure and ophthalmoscopy, as and when required.
    """

    ORTHOPEDICS_CLINIC = "ORTHO"
    """
    Orthopedics clinic.
    """

    HAND_CLINIC = "HAND"
    """
    Hand clinic.
    """

    PAIN_CLINIC = "PAINCL"
    """
    (X12N 261QP3300N)
    """

    PRIMARY_CARE_CLINIC = "PC"
    """
    (X12N 261QP2300N)
    """

    PEDIATRICS_CLINIC = "PEDC"
    """
    Pediatrics clinic.
    """

    PEDIATRIC_RHEUMATOLOGY_CLINIC = "PEDRHEUM"
    """
    Pediatric rheumatology clinic.
    """

    PODIATRY_CLINIC = "POD"
    """
    (X12N 261QP1100N)
    """

    PREVENTIVE_MEDICINE_CLINIC = "PREV"
    """
    Preventive medicine clinic.
    """

    PROCTOLOGY_CLINIC = "PROCTO"
    """
    Proctology clinic.
    """

    PROVIDERS_OFFICE = "PROFF"
    """
    Location where healthcare service was delivered.
    """

    PROSTHODONTICS_CLINIC = "PROS"
    """
    Prosthodontics clinic.
    """

    PSYCHOLOGY_CLINIC = "PSI"
    """
    Psychology clinic.
    """

    PSYCHIATRY_CLINIC = "PSY"
    """
    Psychiatry clinic.
    """

    RHEUMATOLOGY_CLINIC = "RHEUM"
    """
    Rheumatology clinic.
    """

    SPORTS_MEDICINE_CLINIC = "SPMED"
    """
    Sports medicine clinic.
    """

    SURGERY_CLINIC = "SU"
    """
    Surgery clinic.
    """

    PLASTIC_SURGERY_CLINIC = "PLS"
    """
    Plastic surgery clinic.
    """

    UROLOGY_CLINIC = "URO"
    """
    Urology clinic.
    """

    TRANSPLANT_CLINIC = "TR"
    """
    Transplant clinic.
    """

    TRAVEL_AND_GEOGRAPHIC_MEDICINE_CLINIC = "TRAVEL"
    """
    Travel and geographic medicine clinic.
    """

    WOUND_CLINIC = "WND"
    """
    Wound clinic.
    """

    RESIDENTIAL_TREATMENT_FACILITY = "RTF"
    """
    Residential treatment facility.
    """

    PAIN_REHABILITATION_CENTER = "PRC"
    """
    Pain rehabilitation center.
    """

    SUBSTANCE_USE_REHABILITATION_FACILITY = "SURF"
    """
    (X12N 324500000N)
    """

    DELIVERY_ADDRESS = "DADDR"
    """
    Location address where medical supplies were transported to for use.
    """

    MOBILE_UNIT = "MOBL"
    """
    Location (mobile) where healthcare service was delivered.
    """

    AMBULANCE = "AMB"
    """
    Location (mobile) where healthcare service was delivered, identified specifically as an ambulance.
    """

    PHARMACY = "PHARM"
    """
    Location where healthcare service was delivered.
    """

    ACCIDENT_SITE = "ACC"
    """
    Location of an accident where healthcare service was delivered.
    """

    COMMUNITY_LOCATION = "COMM"
    """
    Community location where healthcare is delivered.
    """

    COMMUNITY_SERVICE_CENTER = "CSC"
    """
    A location that plays the role of delivering services which may include providing front-line services to the population of a defined geographic area such as: healthcare services and social services.
    """

    PATIENTS_RESIDENCE = "PTRES"
    """
    location where healthcare was delivered which is the residence of the Patient.
    """

    SCHOOL = "SCHOOL"
    """
    Location where healthcare service was delivered.
    """

    UNDERAGE_PROTECTION_CENTER = "UPC"
    """
    A location that plays the role of delivering services which may include: social emergency services required for a young person as required under any jurisdictional youth laws.
    """

    WORK_SITE = "WORK"
    """
    Location where healthcare service was delivered.
    """

    def visit(
        self,
        diagnostics_or_therapeutics_unit: typing.Callable[[], T_Result],
        cardiovascular_diagnostics_or_therapeutics_unit: typing.Callable[[], T_Result],
        cardiac_catheterization_lab: typing.Callable[[], T_Result],
        echocardiography_lab: typing.Callable[[], T_Result],
        gastroenterology_diagnostics_or_therapeutics_lab: typing.Callable[[], T_Result],
        endoscopy_lab: typing.Callable[[], T_Result],
        radiology_diagnostics_or_therapeutics_unit: typing.Callable[[], T_Result],
        radiation_oncology_unit: typing.Callable[[], T_Result],
        neuroradiology_unit: typing.Callable[[], T_Result],
        hospital: typing.Callable[[], T_Result],
        chronic_care_facility: typing.Callable[[], T_Result],
        hospitals_general_acute_care_hospital: typing.Callable[[], T_Result],
        military_hospital: typing.Callable[[], T_Result],
        psychatric_care_facility: typing.Callable[[], T_Result],
        rehabilitation_hospital: typing.Callable[[], T_Result],
        addiction_treatment_center: typing.Callable[[], T_Result],
        intellectual_impairment_center: typing.Callable[[], T_Result],
        parents_with_adjustment_difficulties_center: typing.Callable[[], T_Result],
        physical_impairment_center: typing.Callable[[], T_Result],
        physical_impairment_hearing_center: typing.Callable[[], T_Result],
        physical_impairment_motor_skills_center: typing.Callable[[], T_Result],
        physical_impairment_visual_skills_center: typing.Callable[[], T_Result],
        youths_with_adjustment_difficulties_center: typing.Callable[[], T_Result],
        hospital_unit: typing.Callable[[], T_Result],
        bone_marrow_transplant_unit: typing.Callable[[], T_Result],
        coronary_care_unit: typing.Callable[[], T_Result],
        chest_unit: typing.Callable[[], T_Result],
        epilepsy_unit: typing.Callable[[], T_Result],
        emergency_room: typing.Callable[[], T_Result],
        emergency_trauma_unit: typing.Callable[[], T_Result],
        hemodialysis_unit: typing.Callable[[], T_Result],
        hospital_laboratory: typing.Callable[[], T_Result],
        inpatient_laboratory: typing.Callable[[], T_Result],
        outpatient_laboratory: typing.Callable[[], T_Result],
        radiology_unit: typing.Callable[[], T_Result],
        specimen_collection_site: typing.Callable[[], T_Result],
        intensive_care_unit: typing.Callable[[], T_Result],
        pediatric_intensive_care_unit: typing.Callable[[], T_Result],
        pediatric_neonatal_intensive_care_unit: typing.Callable[[], T_Result],
        inpatient_pharmacy: typing.Callable[[], T_Result],
        medical_laboratory: typing.Callable[[], T_Result],
        neurology_critical_care_and_stroke_unit: typing.Callable[[], T_Result],
        neurosurgery_unit: typing.Callable[[], T_Result],
        outpatient_pharmacy: typing.Callable[[], T_Result],
        pediatric_unit: typing.Callable[[], T_Result],
        psychiatric_hospital_unit: typing.Callable[[], T_Result],
        rehabilitation_hospital_unit: typing.Callable[[], T_Result],
        sleep_disorders_unit: typing.Callable[[], T_Result],
        nursing_or_custodial_care_facility: typing.Callable[[], T_Result],
        skilled_nursing_facility: typing.Callable[[], T_Result],
        outpatient_facility: typing.Callable[[], T_Result],
        allergy_clinic: typing.Callable[[], T_Result],
        amputee_clinic: typing.Callable[[], T_Result],
        bone_marrow_transplant_clinic: typing.Callable[[], T_Result],
        breast_clinic: typing.Callable[[], T_Result],
        child_and_adolescent_neurology_clinic: typing.Callable[[], T_Result],
        child_and_adolescent_psychiatry_clinic: typing.Callable[[], T_Result],
        ambulatory_health_care_facilities_clinic_center_rehabilitation_cardiac_facilities: typing.Callable[
            [], T_Result
        ],
        pediatric_cardiology_clinic: typing.Callable[[], T_Result],
        coagulation_clinic: typing.Callable[[], T_Result],
        colon_and_rectal_surgery_clinic: typing.Callable[[], T_Result],
        dermatology_clinic: typing.Callable[[], T_Result],
        endocrinology_clinic: typing.Callable[[], T_Result],
        pediatric_endocrinology_clinic: typing.Callable[[], T_Result],
        otorhinolaryngology_clinic: typing.Callable[[], T_Result],
        family_medicine_clinic: typing.Callable[[], T_Result],
        gastroenterology_clinic: typing.Callable[[], T_Result],
        pediatric_gastroenterology_clinic: typing.Callable[[], T_Result],
        general_internal_medicine_clinic: typing.Callable[[], T_Result],
        gynecology_clinic: typing.Callable[[], T_Result],
        hematology_clinic: typing.Callable[[], T_Result],
        pediatric_hematology_clinic: typing.Callable[[], T_Result],
        hypertension_clinic: typing.Callable[[], T_Result],
        impairment_evaluation_center: typing.Callable[[], T_Result],
        infectious_disease_clinic: typing.Callable[[], T_Result],
        pediatric_infectious_disease_clinic: typing.Callable[[], T_Result],
        infertility_clinic: typing.Callable[[], T_Result],
        lympedema_clinic: typing.Callable[[], T_Result],
        medical_genetics_clinic: typing.Callable[[], T_Result],
        nephrology_clinic: typing.Callable[[], T_Result],
        pediatric_nephrology_clinic: typing.Callable[[], T_Result],
        neurology_clinic: typing.Callable[[], T_Result],
        obstetrics_clinic: typing.Callable[[], T_Result],
        oral_and_maxillofacial_surgery_clinic: typing.Callable[[], T_Result],
        medical_oncology_clinic: typing.Callable[[], T_Result],
        pediatric_oncology_clinic: typing.Callable[[], T_Result],
        opthalmology_clinic: typing.Callable[[], T_Result],
        optometry_clinic: typing.Callable[[], T_Result],
        orthopedics_clinic: typing.Callable[[], T_Result],
        hand_clinic: typing.Callable[[], T_Result],
        pain_clinic: typing.Callable[[], T_Result],
        primary_care_clinic: typing.Callable[[], T_Result],
        pediatrics_clinic: typing.Callable[[], T_Result],
        pediatric_rheumatology_clinic: typing.Callable[[], T_Result],
        podiatry_clinic: typing.Callable[[], T_Result],
        preventive_medicine_clinic: typing.Callable[[], T_Result],
        proctology_clinic: typing.Callable[[], T_Result],
        providers_office: typing.Callable[[], T_Result],
        prosthodontics_clinic: typing.Callable[[], T_Result],
        psychology_clinic: typing.Callable[[], T_Result],
        psychiatry_clinic: typing.Callable[[], T_Result],
        rheumatology_clinic: typing.Callable[[], T_Result],
        sports_medicine_clinic: typing.Callable[[], T_Result],
        surgery_clinic: typing.Callable[[], T_Result],
        plastic_surgery_clinic: typing.Callable[[], T_Result],
        urology_clinic: typing.Callable[[], T_Result],
        transplant_clinic: typing.Callable[[], T_Result],
        travel_and_geographic_medicine_clinic: typing.Callable[[], T_Result],
        wound_clinic: typing.Callable[[], T_Result],
        residential_treatment_facility: typing.Callable[[], T_Result],
        pain_rehabilitation_center: typing.Callable[[], T_Result],
        substance_use_rehabilitation_facility: typing.Callable[[], T_Result],
        delivery_address: typing.Callable[[], T_Result],
        mobile_unit: typing.Callable[[], T_Result],
        ambulance: typing.Callable[[], T_Result],
        pharmacy: typing.Callable[[], T_Result],
        accident_site: typing.Callable[[], T_Result],
        community_location: typing.Callable[[], T_Result],
        community_service_center: typing.Callable[[], T_Result],
        patients_residence: typing.Callable[[], T_Result],
        school: typing.Callable[[], T_Result],
        underage_protection_center: typing.Callable[[], T_Result],
        work_site: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT:
            return diagnostics_or_therapeutics_unit()
        if self is ServiceFacilityType.CARDIOVASCULAR_DIAGNOSTICS_OR_THERAPEUTICS_UNIT:
            return cardiovascular_diagnostics_or_therapeutics_unit()
        if self is ServiceFacilityType.CARDIAC_CATHETERIZATION_LAB:
            return cardiac_catheterization_lab()
        if self is ServiceFacilityType.ECHOCARDIOGRAPHY_LAB:
            return echocardiography_lab()
        if self is ServiceFacilityType.GASTROENTEROLOGY_DIAGNOSTICS_OR_THERAPEUTICS_LAB:
            return gastroenterology_diagnostics_or_therapeutics_lab()
        if self is ServiceFacilityType.ENDOSCOPY_LAB:
            return endoscopy_lab()
        if self is ServiceFacilityType.RADIOLOGY_DIAGNOSTICS_OR_THERAPEUTICS_UNIT:
            return radiology_diagnostics_or_therapeutics_unit()
        if self is ServiceFacilityType.RADIATION_ONCOLOGY_UNIT:
            return radiation_oncology_unit()
        if self is ServiceFacilityType.NEURORADIOLOGY_UNIT:
            return neuroradiology_unit()
        if self is ServiceFacilityType.HOSPITAL:
            return hospital()
        if self is ServiceFacilityType.CHRONIC_CARE_FACILITY:
            return chronic_care_facility()
        if self is ServiceFacilityType.HOSPITALS_GENERAL_ACUTE_CARE_HOSPITAL:
            return hospitals_general_acute_care_hospital()
        if self is ServiceFacilityType.MILITARY_HOSPITAL:
            return military_hospital()
        if self is ServiceFacilityType.PSYCHATRIC_CARE_FACILITY:
            return psychatric_care_facility()
        if self is ServiceFacilityType.REHABILITATION_HOSPITAL:
            return rehabilitation_hospital()
        if self is ServiceFacilityType.ADDICTION_TREATMENT_CENTER:
            return addiction_treatment_center()
        if self is ServiceFacilityType.INTELLECTUAL_IMPAIRMENT_CENTER:
            return intellectual_impairment_center()
        if self is ServiceFacilityType.PARENTS_WITH_ADJUSTMENT_DIFFICULTIES_CENTER:
            return parents_with_adjustment_difficulties_center()
        if self is ServiceFacilityType.PHYSICAL_IMPAIRMENT_CENTER:
            return physical_impairment_center()
        if self is ServiceFacilityType.PHYSICAL_IMPAIRMENT_HEARING_CENTER:
            return physical_impairment_hearing_center()
        if self is ServiceFacilityType.PHYSICAL_IMPAIRMENT_MOTOR_SKILLS_CENTER:
            return physical_impairment_motor_skills_center()
        if self is ServiceFacilityType.PHYSICAL_IMPAIRMENT_VISUAL_SKILLS_CENTER:
            return physical_impairment_visual_skills_center()
        if self is ServiceFacilityType.YOUTHS_WITH_ADJUSTMENT_DIFFICULTIES_CENTER:
            return youths_with_adjustment_difficulties_center()
        if self is ServiceFacilityType.HOSPITAL_UNIT:
            return hospital_unit()
        if self is ServiceFacilityType.BONE_MARROW_TRANSPLANT_UNIT:
            return bone_marrow_transplant_unit()
        if self is ServiceFacilityType.CORONARY_CARE_UNIT:
            return coronary_care_unit()
        if self is ServiceFacilityType.CHEST_UNIT:
            return chest_unit()
        if self is ServiceFacilityType.EPILEPSY_UNIT:
            return epilepsy_unit()
        if self is ServiceFacilityType.EMERGENCY_ROOM:
            return emergency_room()
        if self is ServiceFacilityType.EMERGENCY_TRAUMA_UNIT:
            return emergency_trauma_unit()
        if self is ServiceFacilityType.HEMODIALYSIS_UNIT:
            return hemodialysis_unit()
        if self is ServiceFacilityType.HOSPITAL_LABORATORY:
            return hospital_laboratory()
        if self is ServiceFacilityType.INPATIENT_LABORATORY:
            return inpatient_laboratory()
        if self is ServiceFacilityType.OUTPATIENT_LABORATORY:
            return outpatient_laboratory()
        if self is ServiceFacilityType.RADIOLOGY_UNIT:
            return radiology_unit()
        if self is ServiceFacilityType.SPECIMEN_COLLECTION_SITE:
            return specimen_collection_site()
        if self is ServiceFacilityType.INTENSIVE_CARE_UNIT:
            return intensive_care_unit()
        if self is ServiceFacilityType.PEDIATRIC_INTENSIVE_CARE_UNIT:
            return pediatric_intensive_care_unit()
        if self is ServiceFacilityType.PEDIATRIC_NEONATAL_INTENSIVE_CARE_UNIT:
            return pediatric_neonatal_intensive_care_unit()
        if self is ServiceFacilityType.INPATIENT_PHARMACY:
            return inpatient_pharmacy()
        if self is ServiceFacilityType.MEDICAL_LABORATORY:
            return medical_laboratory()
        if self is ServiceFacilityType.NEUROLOGY_CRITICAL_CARE_AND_STROKE_UNIT:
            return neurology_critical_care_and_stroke_unit()
        if self is ServiceFacilityType.NEUROSURGERY_UNIT:
            return neurosurgery_unit()
        if self is ServiceFacilityType.OUTPATIENT_PHARMACY:
            return outpatient_pharmacy()
        if self is ServiceFacilityType.PEDIATRIC_UNIT:
            return pediatric_unit()
        if self is ServiceFacilityType.PSYCHIATRIC_HOSPITAL_UNIT:
            return psychiatric_hospital_unit()
        if self is ServiceFacilityType.REHABILITATION_HOSPITAL_UNIT:
            return rehabilitation_hospital_unit()
        if self is ServiceFacilityType.SLEEP_DISORDERS_UNIT:
            return sleep_disorders_unit()
        if self is ServiceFacilityType.NURSING_OR_CUSTODIAL_CARE_FACILITY:
            return nursing_or_custodial_care_facility()
        if self is ServiceFacilityType.SKILLED_NURSING_FACILITY:
            return skilled_nursing_facility()
        if self is ServiceFacilityType.OUTPATIENT_FACILITY:
            return outpatient_facility()
        if self is ServiceFacilityType.ALLERGY_CLINIC:
            return allergy_clinic()
        if self is ServiceFacilityType.AMPUTEE_CLINIC:
            return amputee_clinic()
        if self is ServiceFacilityType.BONE_MARROW_TRANSPLANT_CLINIC:
            return bone_marrow_transplant_clinic()
        if self is ServiceFacilityType.BREAST_CLINIC:
            return breast_clinic()
        if self is ServiceFacilityType.CHILD_AND_ADOLESCENT_NEUROLOGY_CLINIC:
            return child_and_adolescent_neurology_clinic()
        if self is ServiceFacilityType.CHILD_AND_ADOLESCENT_PSYCHIATRY_CLINIC:
            return child_and_adolescent_psychiatry_clinic()
        if (
            self
            is ServiceFacilityType.AMBULATORY_HEALTH_CARE_FACILITIES_CLINIC_CENTER_REHABILITATION_CARDIAC_FACILITIES
        ):
            return ambulatory_health_care_facilities_clinic_center_rehabilitation_cardiac_facilities()
        if self is ServiceFacilityType.PEDIATRIC_CARDIOLOGY_CLINIC:
            return pediatric_cardiology_clinic()
        if self is ServiceFacilityType.COAGULATION_CLINIC:
            return coagulation_clinic()
        if self is ServiceFacilityType.COLON_AND_RECTAL_SURGERY_CLINIC:
            return colon_and_rectal_surgery_clinic()
        if self is ServiceFacilityType.DERMATOLOGY_CLINIC:
            return dermatology_clinic()
        if self is ServiceFacilityType.ENDOCRINOLOGY_CLINIC:
            return endocrinology_clinic()
        if self is ServiceFacilityType.PEDIATRIC_ENDOCRINOLOGY_CLINIC:
            return pediatric_endocrinology_clinic()
        if self is ServiceFacilityType.OTORHINOLARYNGOLOGY_CLINIC:
            return otorhinolaryngology_clinic()
        if self is ServiceFacilityType.FAMILY_MEDICINE_CLINIC:
            return family_medicine_clinic()
        if self is ServiceFacilityType.GASTROENTEROLOGY_CLINIC:
            return gastroenterology_clinic()
        if self is ServiceFacilityType.PEDIATRIC_GASTROENTEROLOGY_CLINIC:
            return pediatric_gastroenterology_clinic()
        if self is ServiceFacilityType.GENERAL_INTERNAL_MEDICINE_CLINIC:
            return general_internal_medicine_clinic()
        if self is ServiceFacilityType.GYNECOLOGY_CLINIC:
            return gynecology_clinic()
        if self is ServiceFacilityType.HEMATOLOGY_CLINIC:
            return hematology_clinic()
        if self is ServiceFacilityType.PEDIATRIC_HEMATOLOGY_CLINIC:
            return pediatric_hematology_clinic()
        if self is ServiceFacilityType.HYPERTENSION_CLINIC:
            return hypertension_clinic()
        if self is ServiceFacilityType.IMPAIRMENT_EVALUATION_CENTER:
            return impairment_evaluation_center()
        if self is ServiceFacilityType.INFECTIOUS_DISEASE_CLINIC:
            return infectious_disease_clinic()
        if self is ServiceFacilityType.PEDIATRIC_INFECTIOUS_DISEASE_CLINIC:
            return pediatric_infectious_disease_clinic()
        if self is ServiceFacilityType.INFERTILITY_CLINIC:
            return infertility_clinic()
        if self is ServiceFacilityType.LYMPEDEMA_CLINIC:
            return lympedema_clinic()
        if self is ServiceFacilityType.MEDICAL_GENETICS_CLINIC:
            return medical_genetics_clinic()
        if self is ServiceFacilityType.NEPHROLOGY_CLINIC:
            return nephrology_clinic()
        if self is ServiceFacilityType.PEDIATRIC_NEPHROLOGY_CLINIC:
            return pediatric_nephrology_clinic()
        if self is ServiceFacilityType.NEUROLOGY_CLINIC:
            return neurology_clinic()
        if self is ServiceFacilityType.OBSTETRICS_CLINIC:
            return obstetrics_clinic()
        if self is ServiceFacilityType.ORAL_AND_MAXILLOFACIAL_SURGERY_CLINIC:
            return oral_and_maxillofacial_surgery_clinic()
        if self is ServiceFacilityType.MEDICAL_ONCOLOGY_CLINIC:
            return medical_oncology_clinic()
        if self is ServiceFacilityType.PEDIATRIC_ONCOLOGY_CLINIC:
            return pediatric_oncology_clinic()
        if self is ServiceFacilityType.OPTHALMOLOGY_CLINIC:
            return opthalmology_clinic()
        if self is ServiceFacilityType.OPTOMETRY_CLINIC:
            return optometry_clinic()
        if self is ServiceFacilityType.ORTHOPEDICS_CLINIC:
            return orthopedics_clinic()
        if self is ServiceFacilityType.HAND_CLINIC:
            return hand_clinic()
        if self is ServiceFacilityType.PAIN_CLINIC:
            return pain_clinic()
        if self is ServiceFacilityType.PRIMARY_CARE_CLINIC:
            return primary_care_clinic()
        if self is ServiceFacilityType.PEDIATRICS_CLINIC:
            return pediatrics_clinic()
        if self is ServiceFacilityType.PEDIATRIC_RHEUMATOLOGY_CLINIC:
            return pediatric_rheumatology_clinic()
        if self is ServiceFacilityType.PODIATRY_CLINIC:
            return podiatry_clinic()
        if self is ServiceFacilityType.PREVENTIVE_MEDICINE_CLINIC:
            return preventive_medicine_clinic()
        if self is ServiceFacilityType.PROCTOLOGY_CLINIC:
            return proctology_clinic()
        if self is ServiceFacilityType.PROVIDERS_OFFICE:
            return providers_office()
        if self is ServiceFacilityType.PROSTHODONTICS_CLINIC:
            return prosthodontics_clinic()
        if self is ServiceFacilityType.PSYCHOLOGY_CLINIC:
            return psychology_clinic()
        if self is ServiceFacilityType.PSYCHIATRY_CLINIC:
            return psychiatry_clinic()
        if self is ServiceFacilityType.RHEUMATOLOGY_CLINIC:
            return rheumatology_clinic()
        if self is ServiceFacilityType.SPORTS_MEDICINE_CLINIC:
            return sports_medicine_clinic()
        if self is ServiceFacilityType.SURGERY_CLINIC:
            return surgery_clinic()
        if self is ServiceFacilityType.PLASTIC_SURGERY_CLINIC:
            return plastic_surgery_clinic()
        if self is ServiceFacilityType.UROLOGY_CLINIC:
            return urology_clinic()
        if self is ServiceFacilityType.TRANSPLANT_CLINIC:
            return transplant_clinic()
        if self is ServiceFacilityType.TRAVEL_AND_GEOGRAPHIC_MEDICINE_CLINIC:
            return travel_and_geographic_medicine_clinic()
        if self is ServiceFacilityType.WOUND_CLINIC:
            return wound_clinic()
        if self is ServiceFacilityType.RESIDENTIAL_TREATMENT_FACILITY:
            return residential_treatment_facility()
        if self is ServiceFacilityType.PAIN_REHABILITATION_CENTER:
            return pain_rehabilitation_center()
        if self is ServiceFacilityType.SUBSTANCE_USE_REHABILITATION_FACILITY:
            return substance_use_rehabilitation_facility()
        if self is ServiceFacilityType.DELIVERY_ADDRESS:
            return delivery_address()
        if self is ServiceFacilityType.MOBILE_UNIT:
            return mobile_unit()
        if self is ServiceFacilityType.AMBULANCE:
            return ambulance()
        if self is ServiceFacilityType.PHARMACY:
            return pharmacy()
        if self is ServiceFacilityType.ACCIDENT_SITE:
            return accident_site()
        if self is ServiceFacilityType.COMMUNITY_LOCATION:
            return community_location()
        if self is ServiceFacilityType.COMMUNITY_SERVICE_CENTER:
            return community_service_center()
        if self is ServiceFacilityType.PATIENTS_RESIDENCE:
            return patients_residence()
        if self is ServiceFacilityType.SCHOOL:
            return school()
        if self is ServiceFacilityType.UNDERAGE_PROTECTION_CENTER:
            return underage_protection_center()
        if self is ServiceFacilityType.WORK_SITE:
            return work_site()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...commons.types.email import Email
from ...commons.types.phone_number import PhoneNumber
from .individual_id import IndividualId
from .patient_base import PatientBase


class Patient(PatientBase):
    """
    Examples
    --------
    import uuid

    from candid import (
        Gender,
        Patient,
        PhoneNumber,
        PhoneNumberType,
        State,
        StreetAddressShortZip,
    )

    Patient(
        individual_id=uuid.UUID(
            "93ddbebf-4956-4482-9a6c-21499b7e4e5d",
        ),
        phone_numbers=[
            PhoneNumber(
                number="1234567890",
                type=PhoneNumberType.HOME,
            )
        ],
        phone_consent=True,
        email="johndoe@joincandidhealth.com",
        email_consent=True,
        external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
        date_of_birth="2000-01-01",
        address=StreetAddressShortZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
        first_name="John",
        last_name="Doe",
        gender=Gender.MALE,
    )
    """

    individual_id: IndividualId
    phone_numbers: typing.List[PhoneNumber]
    phone_consent: bool
    email: typing.Optional[Email] = None
    email_consent: bool

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}
